@@ -1,6 +1,6 @@
 /* MAPIStoreFallbackContext.h - this file is part of SOGo
  *
- * Copyright (C) 2010 Inverse inc.
+ * Copyright (C) 2010-2012 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -23,9 +23,9 @@
 #ifndef MAPISTOREFALLBACKCONTEXT_H
 #define MAPISTOREFALLBACKCONTEXT_H
 
-#import "MAPIStoreFSBaseContext.h"
+#import "MAPIStoreDBBaseContext.h"
 
-@interface MAPIStoreFallbackContext : MAPIStoreFSBaseContext
+@interface MAPIStoreFallbackContext : MAPIStoreDBBaseContext
 
 @end
 
