@@ -259,7 +259,6 @@ static Class NSNumberK;
   }
 }
  */
-
 - (void) _setChangeKey: (NSData *) changeKey
        forMessageEntry: (NSMutableDictionary *) messageEntry
       inChangeListOnly: (BOOL) inChangeListOnly
@@ -461,7 +460,10 @@ static Class NSNumberK;
               [mapping setObject: cLastModified forKey: changeNumber];
               [messageEntry setObject: changeNumber forKey: @"version"];
 
-              newChangeNum = [changeNumber unsignedLongValue];
+              newChangeNum = [changeNumber unsignedLongLongValue];
+	      
+	      // A GLOBCNT structure is a 6-byte global namespace counter,
+	      // we strip the first 2 bytes. The first two bytes is the ReplicaId
               changeKey = [self getReplicaKeyFromGlobCnt: newChangeNum >> 16];
               [self _setChangeKey: changeKey forMessageEntry: messageEntry
                  inChangeListOnly: NO];
