@@ -694,10 +694,18 @@ const unsigned short ansicpg874[256] = {
             }
           else if (*(_bytes+1) == '*')
             {
-              while (*_bytes != '}')
+	      int cc = 1;
+	      
+	      do
                 {
+                  if (*_bytes == '{')
+                    cc++;
+                  if (*_bytes == '}')
+                    cc--;
+                  
                   ADVANCE;
                 }
+              while (cc != 0);
 
               continue;
             }
@@ -777,6 +785,10 @@ const unsigned short ansicpg874[256] = {
               [_html appendBytes: v  length: strlen(v)];
               free(v);
             }
+          else if ([s hasPrefix: @"fcs"])
+            {
+              // ignore
+            }
           else if ([s hasPrefix: @"fs"])
             {
               // ignore
@@ -789,6 +801,10 @@ const unsigned short ansicpg874[256] = {
             {
               // ignore
             }
+         else if ([s hasPrefix: @"fromtext"])
+            {
+              // ignore
+            }
           else if ([s hasPrefix: @"f"] && [s length] > 1)
             {
               RTFFontInfo *fontInfo;
