@@ -28,6 +28,8 @@
 @class NSException;
 @class NSString;
 
+@class WORequest;
+
 @class iCalEvent;
 @class iCalCalendar;
 
@@ -49,6 +51,9 @@
 - (NSArray *) postCalDAVEventReplyTo: (NSArray *) recipients  from: (NSString *) originator;
 - (NSArray *) postCalDAVEventCancelTo: (NSArray *) recipients  from: (NSString *) originator;
 
+- (NSException *) updateContentWithCalendar: (iCalCalendar *) calendar
+                                fromRequest: (WORequest *) rq;
+
 @end
 
 #endif /* __Appointments_SOGoAppointmentObject_H__ */
