@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2007-2011 Inverse inc.
+  Copyright (C) 2007-2013 Inverse inc.
 
   This file is part of SOGo.
 
@@ -26,6 +26,9 @@
 #import <Foundation/NSString.h>
 #import <Foundation/NSValue.h>
 
+#import <DOM/DOMElement.h>
+#import <DOM/DOMProtocols.h>
+
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/SoHTTPAuthenticator.h>
 #import <NGObjWeb/WORequest.h>
@@ -56,6 +59,8 @@
 
 #import "SOGoMailAccount.h"
 
+#define XMLNS_INVERSEDAV @"urn:inverse:params:xml:ns:inverse-dav"
+
 @implementation SOGoMailAccount
 
 static NSString *inboxFolderName = @"INBOX";
