@@ -1,15 +1,15 @@
 /*
-  Copyright (C) 2009-2011 Inverse inc.
+  Copyright (C) 2009-2013 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
-  This file is part of OpenGroupware.org.
+  This file is part of SOGo.
 
-  OGo is free software; you can redistribute it and/or modify it under
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
 
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
@@ -1177,6 +1177,10 @@ static NSString *defaultUserID =  @"anyone";
   if ([uid hasPrefix: @"@"])
     return [[[[context activeUser] domainDefaults] imapAclGroupIdPrefix]
              stringByAppendingString: [uid substringFromIndex: 1]];
+  else if ([[[context activeUser] domainDefaults] forceExternalLoginWithEmail])
+    {
+      return [[[SOGoUser userWithLogin: uid] primaryIdentity] objectForKey: @"email"];
+    }
   else
     return uid;
 }
