@@ -1,6 +1,6 @@
 /* SOGoCache.m - this file is part of SOGo
  *
- * Copyright (C) 2008-2010 Inverse inc.
+ * Copyright (C) 2008-2013 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *         Ludovic Marcotte <lmarcotte@inverse.ca>
@@ -24,22 +24,24 @@
 /*
  * [ Cache Structure ]
  *
- * users                 value = instances of SOGoUser > flushed after the completion of every SOGo requests
- * groups                value = instances of SOGoGroup > flushed after the completion of every SOGo requests
- * imap4Connections      value = 
- * localCache            value = any value of what's in memcached - this is used to NOT query memcached within the same sogod instance
+ * users                    value = instances of SOGoUser > flushed after the completion of every SOGo requests
+ * groups                   value = instances of SOGoGroup > flushed after the completion of every SOGo requests
+ * imap4Connections         value = 
+ * localCache               value = any value of what's in memcached - this is used to NOT query memcached within the same sogod instance
  * 
  * [ Distributed (using memcached) cache structure ]
  *
- * <uid>+defaults        value = NSDictionary instance > user's defaults
- * <uid>+settings        value = NSDictionary instance > user's settings
- * <uid>+attributes      value = NSMutableDictionary instance > user's LDAP attributes
- * <object path>+acl     value = NSDictionary instance > ACLs on an object at specified path
- * <groupname>+<domain>  value = NSString instance (array components separated by ",") or group member logins for a specific group in domain
- * cas-id:< >            value = 
- * cas-ticket:< >        value =
- * cas-pgtiou:< >        value =
- * session:< >           value =
+ * <uid>+defaults           value = NSDictionary instance > user's defaults
+ * <uid>+settings           value = NSDictionary instance > user's settings
+ * <uid>+attributes         value = NSMutableDictionary instance > user's LDAP attributes
+ * <object path>+acl        value = NSDictionary instance > ACLs on an object at specified path
+ * <groupname>+<domain>     value = NSString instance (array components separated by ",") or group member logins for a specific group in domain
+ * cas-id:< >               value = 
+ * cas-ticket:< >           value =
+ * cas-pgtiou:< >           value =
+ * session:< >              value =
+ * <uid>+failedlogins       value = NSDictionary instance holding the failed count and the date of the first failed authentication
+ * <uid>+messagesubmissions value = NSDictionary instance holding the number of messages sent, and number of recipients
  */
 
 
@@ -47,6 +49,7 @@
 #import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSLock.h>
+#import <Foundation/NSValue.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSTimer.h>
 
@@ -327,6 +330,9 @@ static memcached_st *handle = NULL;
                              " no handle exists"), key];
 }
 
+//
+//
+//
 - (void) setValue: (NSString *) value
            forKey: (NSString *) key
 {
@@ -334,6 +340,9 @@ static memcached_st *handle = NULL;
           expire: cleanupInterval];
 }
 
+//
+//
+//
 - (NSString *) valueForKey: (NSString *) key
 {
   NSString *valueString;
@@ -372,6 +381,9 @@ static memcached_st *handle = NULL;
   return valueString;
 }
 
+//
+//
+//
 - (void) removeValueForKey: (NSString *) key
 {
   NSData *keyData;
@@ -420,6 +432,9 @@ static memcached_st *handle = NULL;
     }
 }
 
+//
+//
+//
 - (NSString *) _valuesOfType: (NSString *) theType
                       forKey: (NSString *) theKey
 {
@@ -439,6 +454,9 @@ static memcached_st *handle = NULL;
   return valueString;
 }
 
+//
+//
+//
 - (void) setUserAttributes: (NSString *) theAttributes
                   forLogin: (NSString *) login
 {
@@ -475,6 +493,116 @@ static memcached_st *handle = NULL;
   return [self _valuesOfType: @"settings" forKey: theLogin];
 }
 
+//
+// SOGo password failed counts
+//
+- (void) setFailedCount: (int) theCount
+               forLogin: (NSString *) theLogin
+{
+  NSMutableDictionary *d;
+  NSNumber *count;
+
+  if (theCount)
+    {
+      count = [NSNumber numberWithInt: theCount];
+
+      d = [NSMutableDictionary dictionaryWithDictionary: [self failedCountForLogin: theLogin]];
+
+      if (![d objectForKey: @"InitialDate"])
+        {
+          [d setObject: [NSNumber numberWithUnsignedInt: [[NSCalendarDate date] timeIntervalSince1970]] forKey: @"InitialDate"];
+        }
+      
+      [d setObject: count  forKey: @"FailedCount"];
+      [self _cacheValues: [d jsonRepresentation]
+                  ofType:  @"failedlogins"
+                  forKey: theLogin];
+    }
+  else
+    {
+      [self removeValueForKey: [NSString stringWithFormat: @"%@+failedlogins", theLogin]];
+    }
+}
+
+//
+// Returns a dictionary with two keys/values
+//
+// FailedCount -> 
+// InitialDate ->
+//
+- (NSDictionary *) failedCountForLogin: (NSString *) theLogin
+{
+  NSDictionary *d;
+  NSString *s;
+  
+  s = [self _valuesOfType: @"failedlogins" forKey: theLogin];
+  d = nil;
+
+  if (s)
+    {
+      d = [s objectFromJSONString];
+    }
+  
+  return d;
+}
+
+//
+//
+//
+- (void) setMessageSubmissionsCount: (int) theCount
+                    recipientsCount: (int) theRecipientsCount
+                           forLogin: (NSString *) theLogin
+{
+  NSNumber *messages_count, *recipients_count;
+  NSMutableDictionary *d;
+
+  if (theCount)
+    {
+      messages_count = [NSNumber numberWithInt: theCount];
+      recipients_count = [NSNumber numberWithInt: theRecipientsCount];
+
+      d = [NSMutableDictionary dictionaryWithDictionary: [self messageSubmissionsCountForLogin: theLogin]];
+
+      if (![d objectForKey: @"InitialDate"])
+        {
+          [d setObject: [NSNumber numberWithUnsignedInt: [[NSCalendarDate date] timeIntervalSince1970]] forKey: @"InitialDate"];
+        }
+      
+      [d setObject: messages_count  forKey: @"MessagesCount"];
+      [d setObject: recipients_count  forKey: @"RecipientsCount"];
+      
+      [self _cacheValues: [d jsonRepresentation]
+                  ofType:  @"messagesubmissions"
+                  forKey: theLogin];
+    }
+  else
+    {
+      [self removeValueForKey: [NSString stringWithFormat: @"%@+messagesubmissions", theLogin]];
+    }
+}
+
+//
+// MessagesCount ->
+// RecipientsCount ->
+// InitialDate ->
+//
+- (NSDictionary *) messageSubmissionsCountForLogin: (NSString *) theLogin
+{
+  NSDictionary *d;
+  NSString *s;
+  
+  s = [self _valuesOfType: @"messagesubmissions" forKey: theLogin];
+  d = nil;
+
+  if (s)
+    {
+      d = [s objectFromJSONString];
+    }
+  
+  return d;
+}
+
+
 //
 // CAS session support
 //
@@ -520,6 +648,17 @@ static memcached_st *handle = NULL;
           forKey: [NSString stringWithFormat: @"cas-pgtiou:%@", pgtIou]];
 }
 
+- (void) removeCASSessionWithTicket: (NSString *) ticket
+{
+  NSString *key, *session;
+  if ((session = [self CASSessionWithTicket: ticket]))
+    {
+      key = [NSString stringWithFormat: @"cas-ticket:%@", ticket];
+      [self removeValueForKey: key];
+      [self debugWithFormat: @"Removed session: %@", session];
+    }
+}
+
 // SAML2 support
 - (NSDictionary *) saml2LoginDumpsForIdentifier: (NSString *) identifier
 {
