@@ -51,7 +51,7 @@ static NSMutableDictionary *fieldTypes = nil;
 static NSDictionary *sieveFields = nil;
 static NSDictionary *sieveFlags = nil;
 static NSDictionary *operatorRequirements = nil;
-static NSDictionary *methodRequirements = nil;
+static NSMutableDictionary *methodRequirements = nil;
 static NSString *sieveScriptName = @"sogo";
 
 
@@ -188,16 +188,16 @@ static NSString *sieveScriptName = @"sogo";
   if (!methodRequirements)
     {
       methodRequirements
-        = [NSDictionary dictionaryWithObjectsAndKeys:
-                          @"imapflags", @"addflag",
-                        @"imapflags", @"removeflag",
-                        @"imapflags", @"flag",
-                        @"vacation", @"vacation",
-                        @"notify", @"notify",
-                        @"fileinto", @"fileinto",
-                        @"reject", @"reject",
-                        @"regex", @"regex",
-                        nil];
+        = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                 @"imapflags", @"addflag",
+                               @"imapflags", @"removeflag",
+                               @"imapflags", @"flag",
+                               @"vacation", @"vacation",
+                               @"notify", @"notify",
+                               @"fileinto", @"fileinto",
+                               @"reject", @"reject",
+                               @"regex", @"regex",
+                               nil];
       [methodRequirements retain];
     }
 }
@@ -278,7 +278,7 @@ static NSString *sieveScriptName = @"sogo";
       else
         scriptError
           = [NSString stringWithFormat: @"Rule based on unknown field '%@'",
-                      *field];
+                      jsonField];
     }
   else
     scriptError = @"Rule without any specified field.";
@@ -474,8 +474,8 @@ static NSString *sieveScriptName = @"sogo";
                 sieveAction = [NSString stringWithFormat: @"%@ %@",
                                         method, [argument asSieveQuotedString]];
               else if ([method isEqualToString: @"reject"])
-		sieveAction = [NSString stringWithFormat: @"%@ %@",
-					method, [argument asSieveQuotedString]];
+                sieveAction = [NSString stringWithFormat: @"%@ %@",
+                                method, [argument asSieveQuotedString]];
               else
                 scriptError
                   = [NSString stringWithFormat: @"Action has unknown method '%@'",
@@ -563,7 +563,6 @@ static NSString *sieveScriptName = @"sogo";
   NSString *sieveText;
   NSArray *scripts;
   int count, max;
-  BOOL previousWasConditional;
   NSDictionary *currentScript;
 
   sieveScript = [NSMutableString stringWithCapacity: 8192];
@@ -576,22 +575,12 @@ static NSString *sieveScriptName = @"sogo";
   max = [scripts count];
   if (max)
     {
-      previousWasConditional = NO;
       for (count = 0; !scriptError && count < max; count++)
         {
           currentScript = [scripts objectAtIndex: count];
           if ([[currentScript objectForKey: @"active"] boolValue])
             {
               sieveText = [self _convertScriptToSieve: currentScript];
-              if ([sieveText hasPrefix: @"if"])
-                {
-                  if (previousWasConditional)
-                    [sieveScript appendFormat: @"els"];
-                  else
-                    previousWasConditional = YES;
-                }
-              else
-                previousWasConditional = NO;
               [sieveScript appendString: sieveText];
             }
         }
@@ -616,9 +605,9 @@ static NSString *sieveScriptName = @"sogo";
 //
 //
 - (BOOL) updateFiltersForLogin: (NSString *) theLogin
-		      authname: (NSString *) theAuthName
-		      password: (NSString *) thePassword
-		       account: (SOGoMailAccount *) theAccount
+                      authname: (NSString *) theAuthName
+                      password: (NSString *) thePassword
+                       account: (SOGoMailAccount *) theAccount
 {
   NSMutableArray *req;
   NSMutableString *script, *header;
@@ -626,8 +615,8 @@ static NSString *sieveScriptName = @"sogo";
   SOGoUserDefaults *ud;
   SOGoDomainDefaults *dd;
   NGSieveClient *client;
-  NSString *filterScript, *v, *sieveServer;
-  NSURL *url;
+  NSString *filterScript, *v, *sieveServer, *sieveScheme, *sieveQuery, *imapServer;
+  NSURL *url, *cUrl;
   
   int sievePort;
   BOOL b, connected;
@@ -642,104 +631,9 @@ static NSString *sieveScriptName = @"sogo";
   connected = YES;
   b = NO;
 
-  script = [NSMutableString string];
-
-  // Right now, we handle Sieve filters here and only for vacation
-  // and forwards. Traditional filters support (for fileinto, for
-  // example) will be added later.
-  values = [ud vacationOptions];
-
-  // We handle vacation messages.
-  // See http://ietfreport.isoc.org/idref/draft-ietf-sieve-vacation/
-  if (values && [[values objectForKey: @"enabled"] boolValue])
-    {
-      NSArray *addresses;
-      NSString *text;
-      BOOL ignore;
-      int days, i;
-            
-      days = [[values objectForKey: @"daysBetweenResponse"] intValue];
-      addresses = [values objectForKey: @"autoReplyEmailAddresses"];
-      ignore = [[values objectForKey: @"ignoreLists"] boolValue];
-      text = [values objectForKey: @"autoReplyText"];
-      b = YES;
-
-      if (days == 0)
-	days = 7;
-
-      [req addObjectUniquely: @"vacation"];
-
-      // Skip mailing lists
-      if (ignore)
-	[script appendString: @"if allof ( not exists [\"list-help\", \"list-unsubscribe\", \"list-subscribe\", \"list-owner\", \"list-post\", \"list-archive\", \"list-id\", \"Mailing-List\"], not header :comparator \"i;ascii-casemap\" :is \"Precedence\" [\"list\", \"bulk\", \"junk\"], not header :comparator \"i;ascii-casemap\" :matches \"To\" \"Multiple recipients of*\" ) {"];
-      
-      [script appendFormat: @"vacation :days %d :addresses [", days];
-
-      for (i = 0; i < [addresses count]; i++)
-	{
-	  [script appendFormat: @"\"%@\"", [addresses objectAtIndex: i]];
-	  
-	  if (i == [addresses count]-1)
-	    [script appendString: @"] "];
-	  else
-	    [script appendString: @", "];
-	}
-      
-      [script appendFormat: @"text:\r\n%@\r\n.\r\n;\r\n", text];
-      
-      if (ignore)
-	[script appendString: @"}\r\n"];
-    }
-
-
-  // We handle mail forward
-  values = [ud forwardOptions];
-
-  if (values && [[values objectForKey: @"enabled"] boolValue])
-    {
-      id addresses;
-      int i;
-
-      b = YES;
-      
-      addresses = [values objectForKey: @"forwardAddress"];
-      if ([addresses isKindOfClass: [NSString class]])
-        addresses = [NSArray arrayWithObject: addresses];
-
-      for (i = 0; i < [addresses count]; i++)
-	{
-          v = [addresses objectAtIndex: i];
-          if (v && [v length] > 0)
-            [script appendFormat: @"redirect \"%@\";\r\n", v];
-        }
-      
-      if ([[values objectForKey: @"keepCopy"] boolValue])
-	[script appendString: @"keep;\r\n"];
-    }
   
-  filterScript = [self sieveScriptWithRequirements: req];
-  if (filterScript)
-    {
-      if ([filterScript length])
-        {
-          b = YES;
-          [script appendString: filterScript];
-        }
-    }
-  else
-    {
-      NSLog(@"Sieve generation failure: %@", [self lastScriptError]);
-      return NO;
-    }
-
-  if ([req count])
-    {
-      header = [NSString stringWithFormat: @"require [\"%@\"];\r\n",
-                         [req componentsJoinedByString: @"\",\""]];
-      [script insertString: header  atIndex: 0];
-    }
-
-  // We connect to our Sieve server and upload the script.
+  // We connect to our Sieve server and check capabilities, in order
+  // to generate the right script, based on capabilities
   //
   // sieveServer might have the following format:
   //
@@ -749,36 +643,48 @@ static NSString *sieveScriptName = @"sogo";
   //
   // Values such as "localhost" or "localhost:2000" are NOT supported.
   //
-  sieveServer = [dd sieveServer];
-  sievePort = 2000;
-  url = nil;
-      
-  if (!sieveServer)
-    {
-      NSString *s;
-      
-      s = [dd imapServer];
-      
-      if (s)
-	{
-	  NSURL *url;
-	  
-	  url = [NSURL URLWithString: s];
+  // We first try to get the user's preferred Sieve server
+  sieveServer = [[[user mailAccounts] objectAtIndex: 0] objectForKey: @"sieveServerName"];
+  imapServer = [[[user mailAccounts] objectAtIndex: 0] objectForKey: @"serverName"];
 
-	  if ([url host])
-	    sieveServer = [url host];
-	  else
-	    sieveServer = s;
-	}
-      else
-	sieveServer = @"localhost";
-      
-      url = [NSURL URLWithString: [NSString stringWithFormat: @"%@:%d", sieveServer, sievePort]];
-    }
+  cUrl = [NSURL URLWithString: (sieveServer ? sieveServer : @"")];
+
+  if ([dd sieveServer] && [[dd sieveServer] length] > 0)
+    url = [NSURL URLWithString: [dd sieveServer]];
   else
-    {
-      url = [NSURL URLWithString: sieveServer];
-    }
+    url = [NSURL URLWithString: @"localhost"];
+
+  if ([cUrl host])
+    sieveServer = [cUrl host];
+  if (!sieveServer && [url host])
+    sieveServer = [url host];
+  if (!sieveServer && [dd sieveServer])
+    sieveServer = [dd sieveServer];
+  if (!sieveServer && imapServer)
+    sieveServer = [[NSURL URLWithString: imapServer] host];
+  if (!sieveServer)
+    sieveServer = @"localhost";
+
+  sieveScheme = [cUrl scheme] ? [cUrl scheme] : [url scheme];
+  if (!sieveScheme)
+    sieveScheme = @"sieve";
+
+  if ([cUrl port])
+    sievePort = [[cUrl port] intValue];
+  else
+    if ([url port])
+      sievePort = [[url port] intValue];
+    else
+      sievePort = 2000;
+
+  sieveQuery = [cUrl query] ? [cUrl query] : [url query];
+  if (sieveQuery)
+    sieveQuery = [NSString stringWithFormat: @"/?%@", sieveQuery];
+  else
+    sieveQuery = @"";
+
+  url = [NSURL URLWithString: [NSString stringWithFormat: @"%@://%@:%d%@",
+                               sieveScheme, sieveServer, sievePort, sieveQuery]];
 
   client = [[NGSieveClient alloc] initWithURL: url];
   
@@ -820,6 +726,116 @@ static NSString *sieveScriptName = @"sogo";
     [client closeConnection];
     return NO;
   }
+  
+  // We adjust the "methodRequirements" based on the server's 
+  // capabilities. Cyrus exposes "imapflags" while Dovecot (and
+  // potentially others) expose "imap4flags" as specified in RFC5332
+  if ([client hasCapability: @"imap4flags"])
+    {
+      [methodRequirements setObject: @"imap4flags"  forKey: @"addflag"];
+      [methodRequirements setObject: @"imap4flags"  forKey: @"removeflag"];
+      [methodRequirements setObject: @"imap4flags"  forKey: @"flag"];
+    }
+  
+  //
+  // Now let's generate the script
+  //
+  script = [NSMutableString string];
+
+  // We first handle filters
+  filterScript = [self sieveScriptWithRequirements: req];
+  if (filterScript)
+    {
+      if ([filterScript length])
+        {
+          b = YES;
+          [script appendString: filterScript];
+        }
+    }
+  else
+    {
+      NSLog(@"Sieve generation failure: %@", [self lastScriptError]);
+      [client closeConnection];
+      return NO;
+    }
+
+  // We handle vacation messages.
+  // See http://ietfreport.isoc.org/idref/draft-ietf-sieve-vacation/
+  values = [ud vacationOptions];
+
+  if (values && [[values objectForKey: @"enabled"] boolValue])
+    {
+      NSArray *addresses;
+      NSString *text;
+      BOOL ignore;
+      int days, i;
+            
+      days = [[values objectForKey: @"daysBetweenResponse"] intValue];
+      addresses = [values objectForKey: @"autoReplyEmailAddresses"];
+      ignore = [[values objectForKey: @"ignoreLists"] boolValue];
+      text = [values objectForKey: @"autoReplyText"];
+      b = YES;
+
+      if (days == 0)
+        days = 7;
+
+      [req addObjectUniquely: @"vacation"];
+
+      // Skip mailing lists
+      if (ignore)
+        [script appendString: @"if allof ( not exists [\"list-help\", \"list-unsubscribe\", \"list-subscribe\", \"list-owner\", \"list-post\", \"list-archive\", \"list-id\", \"Mailing-List\"], not header :comparator \"i;ascii-casemap\" :is \"Precedence\" [\"list\", \"bulk\", \"junk\"], not header :comparator \"i;ascii-casemap\" :matches \"To\" \"Multiple recipients of*\" ) {"];
+      
+      [script appendFormat: @"vacation :days %d :addresses [", days];
+
+      for (i = 0; i < [addresses count]; i++)
+        {
+          [script appendFormat: @"\"%@\"", [addresses objectAtIndex: i]];
+	  
+          if (i == [addresses count]-1)
+            [script appendString: @"] "];
+          else
+            [script appendString: @", "];
+        }
+      
+      [script appendFormat: @"text:\r\n%@\r\n.\r\n;\r\n", text];
+      
+      if (ignore)
+        [script appendString: @"}\r\n"];
+    }
+
+
+  // We handle mail forward
+  values = [ud forwardOptions];
+
+  if (values && [[values objectForKey: @"enabled"] boolValue])
+    {
+      id addresses;
+      int i;
+
+      b = YES;
+      
+      addresses = [values objectForKey: @"forwardAddress"];
+      if ([addresses isKindOfClass: [NSString class]])
+        addresses = [NSArray arrayWithObject: addresses];
+
+      for (i = 0; i < [addresses count]; i++)
+        {
+          v = [addresses objectAtIndex: i];
+          if (v && [v length] > 0)
+            [script appendFormat: @"redirect \"%@\";\r\n", v];
+        }
+      
+      if ([[values objectForKey: @"keepCopy"] boolValue])
+        [script appendString: @"keep;\r\n"];
+    }
+  
+  if ([req count])
+    {
+      header = [NSString stringWithFormat: @"require [\"%@\"];\r\n",
+                         [req componentsJoinedByString: @"\",\""]];
+      [script insertString: header  atIndex: 0];
+    }
+
 
   /* We ensure to deactive the current active script since it could prevent
      its deletion from the server. */
@@ -838,19 +854,20 @@ static NSString *sieveScriptName = @"sogo";
       result = [client putScript: sieveScriptName  script: script];
       
       if (![[result valueForKey:@"result"] boolValue]) {
-	NSLog(@"Could not upload Sieve script: %@", result);
-	[client closeConnection];	
-	return NO;
+        NSLog(@"Could not upload Sieve script: %@", result);
+        [client closeConnection];	
+        return NO;
       }
       
       result = [client setActiveScript: sieveScriptName];
       if (![[result valueForKey:@"result"] boolValue]) {
-	NSLog(@"Could not enable Sieve script: %@", result);
-	[client closeConnection];
-	return NO;
+        NSLog(@"Could not enable Sieve script: %@", result);
+        [client closeConnection];
+        return NO;
       }
   }
 
+  [client closeConnection];
   return YES;
 }
 
