@@ -1,6 +1,6 @@
 /* SOGoSystemDefaults.h - this file is part of SOGo
  *
- * Copyright (C) 2009-2011 Inverse inc.
+ * Copyright (C) 2009-2013 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *         Francis Lachapelle <flachapelle@inverse.ca>
@@ -86,6 +86,15 @@
 
 - (BOOL) enablePublicAccess;
 
+- (int) maximumFailedLoginCount;
+- (int) maximumFailedLoginInterval;
+- (int) failedLoginBlockInterval;
+
+- (int) maximumMessageSubmissionCount;
+- (int) maximumRecipientCount;
+- (int) maximumSubmissionInterval;
+- (int) messageSubmissionBlockInterval;
+
 @end
 
 #endif /* SOGOSYSTEMDEFAULTS_H */
