@@ -1,8 +1,6 @@
 /* SOGoUserDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2009-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +24,8 @@
 #import <Foundation/NSString.h>
 #import <Foundation/NSTimeZone.h>
 
+#import <NGImap4/NSString+Imap4.h>
+
 #import "NSString+Utilities.h"
 #import "SOGoDomainDefaults.h"
 #import "SOGoSystemDefaults.h"
@@ -420,7 +420,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 
 - (NSString *) draftsFolderName
 {
-  return [self stringForKey: @"SOGoDraftsFolderName"];
+  return [[self stringForKey: @"SOGoDraftsFolderName"]
+             stringByEncodingImap4FolderName];
 }
 
 - (void) setSentFolderName: (NSString *) newValue
@@ -430,7 +431,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 
 - (NSString *) sentFolderName
 {
-  return [self stringForKey: @"SOGoSentFolderName"];
+  return [[self stringForKey: @"SOGoSentFolderName"]
+             stringByEncodingImap4FolderName];
 }
 
 - (void) setTrashFolderName: (NSString *) newValue
@@ -440,7 +442,8 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 
 - (NSString *) trashFolderName
 {
-  return [self stringForKey: @"SOGoTrashFolderName"];
+  return [[self stringForKey: @"SOGoTrashFolderName"]
+             stringByEncodingImap4FolderName];
 }
 
 - (void) setFirstDayOfWeek: (int) newValue
@@ -733,6 +736,26 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self boolForKey: @"SOGoRemindWithASound"];
 }
 
+//
+// Dictionary of arrays. Example:
+//
+// {
+//   label1 => ("Important", "#FF0000");
+//   label2 => ("Work" "#00FF00");
+//   foo_bar => ("Foo Bar", "#0000FF");
+// }
+//
+- (void) setMailLabelsColors: (NSDictionary *) newValues
+{
+  [self setObject: newValues forKey: @"SOGoMailLabelsColors"];
+}
+
+- (NSDictionary *) mailLabelsColors
+{
+  return [self objectForKey: @"SOGoMailLabelsColors"];
+}
+
+
 - (void) setSieveFilters: (NSArray *) newValue
 {
   [self setObject: newValue forKey: @"SOGoSieveFilters"];
