@@ -158,16 +158,16 @@ static NSData* _sanitizeContent(NSData *theData)
     {
       // We check if we see </head> in which case, we don't do any kind
       // of substitution there after.
-      if (i < len-5)
+      if (i < len-6)
 	{
 	  if ((*bytes == '<') &&
 	      (*(bytes+1) == '/') &&
-	      (*(bytes+1) == 'h' || *(bytes+1) == 'H') &&
-	      (*(bytes+2) == 'e' || *(bytes+2) == 'E') &&
-	      (*(bytes+3) == 'a' || *(bytes+3) == 'A') &&
-	      (*(bytes+4) == 'd' || *(bytes+4) == 'D') &&
-	      (*(bytes+7) == '>'))
-          break;
+	      (*(bytes+2) == 'h' || *(bytes+2) == 'H') &&
+	      (*(bytes+3) == 'e' || *(bytes+3) == 'E') &&
+	      (*(bytes+4) == 'a' || *(bytes+4) == 'A') &&
+	      (*(bytes+5) == 'd' || *(bytes+5) == 'D') &&
+	      (*(bytes+6) == '>'))
+            break;
 	}
       
       // We search for something like :
@@ -189,7 +189,7 @@ static NSData* _sanitizeContent(NSData *theData)
 	      j = 8;
               found_delimiter = YES;
 
-	      while (*(bytes+j) != ' ' && *(bytes+j) != '"')
+	      while (*(bytes+j) != ' ' && *(bytes+j) != '"' && *(bytes+j) != '\'')
 		{
 		  j++;
 		  
@@ -262,7 +262,7 @@ static NSData* _sanitizeContent(NSData *theData)
                   buf = malloc((j+1) * sizeof(char));
                   memset (buf, 0, j+1);
                   memcpy (buf, bytes, j);
-                  found_tag = [NSString stringWithCString: buf encoding: NSASCIIStringEncoding];
+                  found_tag = [NSString stringWithCString: buf encoding: NSUTF8StringEncoding];
                   
                   tags = [VoidTags objectEnumerator];
                   tag = [tags nextObject];
@@ -562,6 +562,39 @@ static NSData* _sanitizeContent(NSData *theData)
                                    && ![value hasPrefix: @"mailto:"]
                                    && ![value hasPrefix: @"#"]);
                 }
+	      else if (
+		       // Mouse Events
+		       [name isEqualToString: @"onclick"] ||
+		       [name isEqualToString: @"ondblclick"] ||
+		       [name isEqualToString: @"onmousedown"] ||
+		       [name isEqualToString: @"onmousemove"] ||
+		       [name isEqualToString: @"onmouseout"] ||
+		       [name isEqualToString: @"onmouseup"] ||
+		       [name isEqualToString: @"onmouseover"] ||
+
+		       // Keyboard Events
+		       [name isEqualToString: @"onkeydown"] ||
+		       [name isEqualToString: @"onkeypress"] ||
+		       [name isEqualToString: @"onkeyup"] ||
+
+		       // Frame/Object Events
+		       [name isEqualToString: @"onabort"] ||
+		       [name isEqualToString: @"onerror"] ||
+		       [name isEqualToString: @"onload"] ||
+		       [name isEqualToString: @"onresize"] ||
+		       [name isEqualToString: @"onscroll"]  ||
+		       [name isEqualToString: @"onunload"] ||
+
+		       // Form Events
+		       [name isEqualToString: @"onblur"] ||
+		       [name isEqualToString: @"onchange"] ||
+		       [name isEqualToString: @"onfocus"] ||
+		       [name isEqualToString: @"onreset"] ||
+		       [name isEqualToString: @"onselect"] ||
+		       [name isEqualToString: @"onsubmit"]) 
+		{
+		  skipAttribute = YES;
+		}
               else
                 value = [_attributes valueAtIndex: count];
               if (!skipAttribute)
