@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2006-2011 Inverse inc.
+  Copyright (C) 2006-2013 Inverse inc.
 
   This file is part of SOGo
 
@@ -753,7 +753,7 @@
       // To
       to = [[message objectForKey: @"envelope"] to];
       if ([to count] > 0)
-	[msg addObject: [addressFormatter stringForArray: to]];
+	[msg addObject: [[addressFormatter stringForArray: to] stringByEscapingHTMLString]];
       else
 	[msg addObject: @""];
 
@@ -778,7 +778,7 @@
       // From
       from = [[message objectForKey: @"envelope"] from];
       if ([from count] > 0)
-	[msg addObject: [addressFormatter stringForArray: from]];
+	[msg addObject: [[addressFormatter stringForArray: from] stringByEscapingHTMLString]];
       else
 	[msg addObject: @""];
       
@@ -861,8 +861,9 @@
 
   flags = [[message objectForKey: @"flags"] objectEnumerator];
   while ((currentFlag = [flags nextObject]))
-    if ([currentFlag hasPrefix: @"$label"])
-      [labels addObject: [currentFlag substringFromIndex: 1]];
+    {
+      [labels addObject: currentFlag];
+    }
   
   return [labels componentsJoinedByString: @" "];
 }
