@@ -2,15 +2,17 @@
 
 /*
   Copyright (C) 2005 SKYRIX Software AG
+  Copyright (C) 2006-2013 Inverse inc.
+  
  
-  This file is part of OpenGroupware.org.
+  This file is part of SOGo.
  
-  OGo is free software; you can redistribute it and/or modify it under
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
  
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
@@ -527,6 +529,11 @@ function onAppointmentEditorLoad() {
         organizer.down("a").on("click", onMailTo);
     }
 
+   var createdBy = $("createdByLabel");
+    if (createdBy && createdBy.down("a")) {
+        createdBy.down("a").on("click", onMailTo);
+    }
+
     // Extend JSON representation of attendees
     attendees = $H(attendees);
     initializeAttendeesHref();
