@@ -32,7 +32,6 @@ var CKBUILDER_CONFIG = {
 		'entities' : 1,
 		'floatingspace' : 1,
 		'wysiwygarea' : 1,
-		'indent' : 1,
 		'link' : 1,
 		'list' : 1,
 		'pastetext' : 1,
@@ -54,9 +53,19 @@ var CKBUILDER_CONFIG = {
 		'scayt' : 1,
 		'format' : 1,
 		'justify' : 1,
-		'wsc' : 1
+		'wsc' : 1,
+		'colordialog' : 1,
+		'contextmenu' : 1,
+		'blockquote' : 1,
+		'indentlist' : 1,
+		'table' : 1,
+		'tabletools' : 1,
+		'tableresize' : 1,
+		'tab' : 1,
+		'div' : 1
 	},
 	languages : {
+		'ar' : 1,
 		'ca' : 1,
 		'cs' : 1,
 		'da' : 1,
@@ -71,12 +80,12 @@ var CKBUILDER_CONFIG = {
 		'no' : 1,
 		'nb' : 1,
 		'pl' : 1,
-		'pt' : 1,
+		'pt-br' : 1,
 		'ru' : 1,
 		'sk' : 1,
 		'es' : 1,
 		'sv' : 1,
 		'uk' : 1,
-		'cy' : 1,
+		'cy' : 1
 	}
 };
\ No newline at end of file
